% Code for Mitchell, Robertson and Wright: the "Chan-Koop-Potter" UC model.

% This code is in 2 parts. 

% Part 1 takes, with the kind permission of Joshua
% Chan, Chan's code (also available at
% http://joshuachan.org/code/code_UC_bounded.html), and estimates what we
% call the "CKP" model. 
% Minor edits have been made to Chan's code, indicated below, to work it through our data; 
% these edits and any ensuing errors are the responsibility of Mitchell, 
% Robertson and Wright

% Part 2 then uses the "output" from the CKP model, as described in
% Mitchell, Robertson and Wright.


% Part 1 (Chan's code and words.)
% % =======================================================================
% % AR trend bound model in Chan, Koop and Potter (2013) 
% %
% % y_t = tau_t + rho_t(tau_t-tau_{t-1}) + u_t, u_t ~ N(0,exp(h_t)),
% % tau_t  = tau_{t-1} + v_t, 
% % a  < tau_t < b, where a and b are fixed constants
% % 0 < rho_t < 1
% % See Chan, J.C.C., Koop, G. and Potter, S.M. (2013). A New Model
% % of Trend Inflation, Journal of Business and Economic Statistics, 31(1),
% % 94-106. DOI:10.1080/07350015.2012.741549
% % (c) 2013, Joshua Chan. Email: joshuacc.chan@gmail.com
% % =======================================================================

clear; clc;
nloop = 40000;
burnin = 10000;
 
 
% Load in MRW data for CPI inflation for the 8 OECD countries. These are
% seasonally adjusted data. Seasonal adjustment is via X12 as implemented
% in EViews 9.5. For the US, SA data are available not just as adjusted by X12
% but as published directly in FRED. 
data = xlsread('SA inflation Eviews 7 countries x12.xlsx', 'B2:j226');  
    
country_id = 9; %1: Germany, 2= France; 3=GB; 4= Greece, 5-Italy; 6=Japan,
%7=US (this series is from FRED and SA via 12), 8= Canada 9=US SA directly
%by FRED not x12

y = data(1:end,country_id)*100;
tid = (1961:.25:2017)';
startd=1961;
y0=y(1);
T = length(y);

% Mitchell, Robertson & Wright insersion: experimentation with priors
for priori = 1:1

%% prior
tau0 = 0; invVtau = 1/5;
tau0=0;
invVrho = 1; 
invVh = 1/5;
nutau0 = 10; Stau0 = .02*(nutau0-1);
nurho0 = 10; Srho0 = .001*(nurho0-1);

% Mitchell, Robertson & Wright insersion: experimentation with priors,
% important in applications beyond the US.
% Mean and sd of sig2_tau given prior choices: 
m_sigtau=sqrt(Stau0/(nutau0-1));
sd_sigtau=sqrt((Stau0^2)/(((nutau0-1)^2)*(nutau0-2)));
% Here you can insert your chosen values for the mean and sd of sig2_tau
% and the implied priors are derived
if priori==1 % CKP priors
    m_sigtau=0.141*1;
sd_sigtau=0.0071;sd_sigtau=sd_sigtau.*1;
end
if priori==2
    m_sigtau=0.141*1;
sd_sigtau=0.0071;sd_sigtau=sd_sigtau.*2;
end
if priori==3
    m_sigtau=0.141*1;
sd_sigtau=0.0071;sd_sigtau=sd_sigtau.*100;
end


m_sig2tau=m_sigtau^2;
nutau0=(m_sig2tau^2 + 2*(sd_sigtau^2))/(sd_sigtau^2);
Stau0=(m_sig2tau*(m_sig2tau^2 + sd_sigtau^2))/sd_sigtau^2;

nuh0 = 10; Sh0 = .05*(nuh0-1);
rhob = 1;
a = -20; b = 100; % a < tau_t < b. 
% In MRW we experiment both with CKP's bounded prior, a=0 and b=5, and relaxing this

rho_a=0.0; rho_b= 1; %  to control prior on rho, AR coeff of cycle. This prior places rho between rho_a and rho_b
rho_bar=(rho_b-rho_a)/2;

%% compute and define a few things
id1 = (1:T)';
id2 = (2:T)';
id3 = (1:T-1)';
Kh =  speye(T,T) - spdiags(ones(T-1,1),-1,T,T);
cH = zeros(2,2);
newnutau = (T-1)/2 + nutau0;
newnurho = (T-1)/2 + nurho0;
newnuh = (T-1)/2 + nuh0;

%% initialize for storage
store_sig = zeros(nloop - burnin,3); 
store_tau = zeros(nloop - burnin,T); 
store_rho = zeros(nloop - burnin,T); 
store_lambda = zeros(nloop - burnin,T-1); 
store_lambda_mrw = zeros(nloop - burnin,T); 
store_h = zeros(nloop - burnin,T);
store_u = zeros(nloop - burnin,T); 
store_v = zeros(nloop - burnin,T); 
counttau = 0; counth = 0; countrho = 0; countsigrho = 0; countsigtau = 0;

%% initialize the Markov chain
sigtau = .01;
sigrho = .001;
sigh = .1;
h = log(var(y))*ones(T,1);
rho = max(abs(y)./max(y)-.1,rho_bar); 

expinvh = exp(-h);
S = sparse(1:T,1:T,[invVtau repmat(1/sigtau,1,T-1)],T,T);
Krho = speye(T) - sparse(2:T,1:(T-1),rho(2:T),T,T); 
mu0 = Krho\[rho(1)*y0; sparse(T-1,1)];
Sy = Krho'*sparse(id1,id1, expinvh,T,T)*Krho;
H =  Sy + Kh'*S*Kh;
cholH = chol(H,'lower');
taut = H\(Sy*(y-mu0));
taut = min(taut,b-.1);    taut = max(taut,a+.1);
flag = 0;
while flag == 0
    tau = taut + cholH'\randn(T,1);
    if  max(tau)<=b && min(tau)>=a
        flag = 1;
    end   
end
sqrtsigtau = sqrt(sigtau);
ystar = y - tau;
Sh = spdiags([invVh; 1/sigh*ones(T-1,1)],0,T,T);
KSKh = Kh'*Sh*Kh;    
s2 = (ystar-rho.*[y0; ystar(1:T-1)]).^2;
errh = 1; ht = h;
while errh> 10^(-3);
    expht = exp(ht);
    sexpht = s2./expht;
    fh = -.5 + .5*sexpht;
    Gh = .5*sexpht;
    Hh = KSKh + spdiags(Gh,0,T,T);
    newht = Hh\(fh+Gh.*ht);
    errh = max(abs(newht-ht));
    ht = newht;          
end 
h = ht + chol(Hh,'lower')'\randn(T,1);

%% MCMC starts here
randn('seed',sum(clock*100)); rand('seed',sum(clock*1000));
disp('Starting MCMC.... ');
disp(' ' );
start_time = clock;

for loop = 1:nloop    
  
      %% sample tau
    Krho = speye(T) - sparse(2:T,1:(T-1),rho(2:T),T,T); 
    mu0 = Krho\[rho(1)*y0; sparse(T-1,1)];
    expinvh = exp(-h);    
    S = sparse(1:T,1:T,[invVtau repmat(1/sigtau,1,T-1)],T,T);
    Sy = Krho'*sparse(id1,id1, expinvh,T,T)*Krho;
    H =  Sy + Kh'*S*Kh;
    taut = H\(Sy*(y-mu0));   %% expand around the mode of the linear model
    taut = min(taut,b-.05);    taut = max(taut,a+.05); 
    cholH = chol(H,'lower'); 
        % AR step
            % compute the constant c
    taustar = taut;    
    u = y-taustar-mu0; v = taut-taustar;
    logc =  -.5*u'*Sy*u -.5*(taustar(1)-tau0)^2*invVtau + ...
        -.5*sum((taustar(2:end)- taustar(1:end-1)).^2)/sigtau + ...
        -sum(log(normcdf((b-taustar(1:end-1))/sqrtsigtau) + ...
        -normcdf((a-taustar(1:end-1))/sqrtsigtau))) + .5*v'*H*v + log(3);
    flag = 0; count = 0;
    while flag == 0 && count < 1000 % give up when count >= 1000
        tauc = taut + cholH'\randn(T,1);
        if  max(tauc) <= b - .01 && min(tauc) >= a + .01
            vc = tauc-taut; uc = y-tauc-mu0;
            alpARc =  -.5*uc'*Sy*uc -.5*(tauc(1)-tau0)^2*invVtau +...
                -.5*sum((tauc(2:end)- tauc(1:end-1)).^2)/sigtau + ...
                -sum(log(normcdf((b-tauc(1:end-1))/sqrtsigtau) + ...
                -normcdf((a-tauc(1:end-1))/sqrtsigtau))) + .5*vc'*H*vc - logc;            
            if alpARc > log(rand)
                flag = 1;
            end
        end
        count = count + 1;
    end      
    if flag == 1;
        v = tau-taut;   u = y-tau-mu0;
        alpAR =  -.5*u'*Sy*u -.5*(tau(1)-tau0)^2*invVtau + ...
            -.5*sum((tau(2:end)- tau(1:end-1)).^2)/sigtau + ...
            - sum(log(normcdf((b-tau(1:end-1))/sqrtsigtau) + ...
            -normcdf((a-tau(1:end-1))/sqrtsigtau))) + .5*v'*H*v - logc;
        if alpAR < 0 
            alpMH = 1;
        elseif alpARc < 0
            alpMH = - alpAR;
        else
            alpMH = alpARc - alpAR;
        end        
        if alpMH > log(rand)
            tau = tauc;
            counttau = counttau + 1;
        end     
    end
    
    %% sample rho
        
    ystar = y - tau;
    XinvSy = [y0; ystar(id3)].*expinvh;
    XinvSyX = XinvSy.*[y0; ystar(id3)]; 
    Hrho = sparse(id1,id1,XinvSyX) + Kh'*sparse(id1,id1,[invVrho repmat(1/sigrho,1,T-1)])*Kh;
    rhot = Hrho\(XinvSy.*ystar);    
    rhot = min(rhot,rho_b-.01);    rhot = max(rhot,rho_a+.01); 
    
    cholHrho = chol(Hrho,'lower'); 
        % AR step
            % compute the constant c
    sqrtsigrho = sqrt(sigrho);
    rhostar = rhot;
    u = ystar - [y0; ystar(id3)].*rhostar; v = rhot-rhostar;
    logc = -.5*(u.*expinvh)'*u -.5*rhostar(1)^2*invVrho + ...
        -.5*sum((rhostar(2:end)-rhostar(1:end-1)).^2)/sigrho + ...
        -sum(log(normcdf((rho_b-rhostar(1:end-1))/sqrtsigrho) + ... 
        -normcdf(rho_a-rhostar(1:end-1)/sqrtsigrho))) + .5*v'*Hrho*v + log(3);     
    flag = 0; count = 0;
    while flag == 0 && count < 1000 % give up when count >= 1000
        rhoc = rhot + cholHrho'\randn(T,1);
        if  max(rhoc)<= rho_b -.005 && min(rhoc)>= rho_a+.005 
            vc = rhoc-rhot; uc =  ystar - [y0; ystar(id3)].*rhoc;
            alpARc =  -.5*(uc.*expinvh)'*uc -.5*rhoc(1)^2*invVrho +...
                -.5*sum((rhoc(2:end)- rhoc(1:end-1)).^2)/sigrho + ...
                -sum(log(normcdf((rho_b-rhoc(1:end-1))/sqrtsigrho) + ...
                -normcdf(rho_a-rhoc(1:end-1)/sqrtsigrho))) + .5*vc'*Hrho*vc - logc;            
            if alpARc > log(rand)
                flag = 1;
            end
        end
        count = count + 1;
    end      
    if flag == 1;
        v = rho-rhot;   u = ystar - [y0; ystar(id3)].*rho;
        alpAR = -.5*(u.*expinvh)'*u -.5*rho(1)^2*invVrho +...
                -.5*sum((rho(2:end)- rho(1:end-1)).^2)/sigrho + ...
                -sum(log(normcdf((rho_b-rho(1:end-1))/sqrtsigrho) + ...
                -normcdf(rho_a-rho(1:end-1)/sqrtsigrho))) + .5*v'*Hrho*v - logc; 
        if alpAR < 0 
            alpMH = 1;
        elseif alpARc < 0
            alpMH = - alpAR;
        else
            alpMH = alpARc - alpAR;
        end        
        if alpMH > log(rand)
            rho = rhoc;
            countrho = countrho + 1;
        end
    end
    
   % MRW insersions to Chan code
    rhoL=rho(1:T-1); 
    lambda=(rho(2:T)-1).*rhoL./(rhoL-1); 
    lambda_mrw=rho(1:T); 
    dy=y(2:T)-y(1:T-1); 
    dy(2:T-1)=lambda(2:T-1).*dy(1:T-2); 
    
    %% sample h
    Sh = spdiags([invVh; 1/sigh*ones(T-1,1)],0,T,T);
    KSKh = Kh'*Sh*Kh;    
    s2 = (ystar-rho.*[y0; ystar(1:T-1)]).^2;
    errh = 1; ht = h;
    while errh> 10^(-3);
        expht = exp(ht);
        sexpht = s2./expht;
        fh = -.5 + .5*sexpht;
        Gh = .5*sexpht;
        Hh = KSKh + spdiags(Gh,0,T,T);
        newht = Hh\(fh+Gh.*ht);
        errh = max(abs(newht-ht));
        ht = newht;          
    end 
    cholHh = chol(Hh,'lower');    
    % AR-step:     
    hstar = ht;
    logc = -.5*hstar'*KSKh*hstar -.5*sum(hstar) - .5*exp(-hstar)'*s2 + log(3);
    flag = 0;
    while flag == 0
        hc = ht + cholHh'\randn(T,1);
        vhc = hc-ht;        
        alpARc = -.5*hc'*KSKh*hc -.5*sum(hc) -.5*exp(-hc)'*s2 + ...
            + .5*vhc'*Hh*vhc - logc;            
        if alpARc > log(rand)
            flag = 1;
        end
    end        
    % MH-step
    vh = h-ht;
    alpAR = -.5*h'*KSKh*h -.5*sum(h) -.5*exp(-h)'*s2 + .5*vh'*Hh*vh - logc;
    if alpAR < 0 
        alpMH = 1;
    elseif alpARc < 0
        alpMH = - alpAR;
    else
        alpMH = alpARc - alpAR;
    end        
    if alpMH > log(rand)
        h = hc;
        counth = counth + 1;
    end 
    
    %% sample sigtau    
    errsigtau = tau(2:end) - tau(1:end-1);
    newStau = Stau0 + sum(errsigtau.^2)/2;
    sigtauc = 1/gamrnd(newnutau, 1./newStau);    
    sqrtsigtauc = sqrt(sigtauc);
    sqrtsigtau = sqrt(sigtau);
    siga = a - tau(1:end-1); 
    sigb = b - tau(1:end-1);    
    alpMH = -sum(log(normcdf(sigb/sqrtsigtauc)-normcdf(siga/sqrtsigtauc))) + ...
        sum(log(normcdf(sigb/sqrtsigtau)-normcdf(siga/sqrtsigtau)));
    if alpMH > log(rand)
        sigtau = sigtauc;         
        sqrtsigtau = sqrtsigtauc;
        countsigtau = countsigtau + 1;
    end
    
    %% sample sigrho    
    errsigrho = rho(2:end) - rho(1:end-1);
    newSrho = Srho0 + sum(errsigrho.^2)/2;
    sigrhoc = 1/gamrnd(newnurho, 1./newSrho);   
    
    sqrtsigrhoc = sqrt(sigrhoc);
    sqrtsigrho = sqrt(sigrho);    
    alpMH = -sum(log(normcdf((1-rho(id1))/sqrtsigrhoc)-normcdf(-rho(id1)/sqrtsigrhoc))) + ...
        sum(log(normcdf((1-rho(id1))/sqrtsigrho)-normcdf(-rho(id1)/sqrtsigrho)));
    if alpMH > log(rand)
        sigrho = sigrhoc;         
        sqrtsigrho = sqrtsigrhoc;
        countsigrho = countsigrho + 1;
    end
    
    %% sample sigh
    errh = h(2:end) - h(1:end-1);
    newS = Sh0 + sum(errh.^2)/2;
    sigh = 1/gamrnd(newnuh,1./newS);    
        
    if loop>burnin
        i = loop-burnin;
        store_tau(i,:) = tau';
        store_rho(i,:) = rho';
        store_h(i,:) = h'; 
        store_sig(i,:) = [sigtau sigrho sigh];
        % MRW insersions to Chan code
        store_lambda(i,:) = lambda'; 
        store_lambda_mrw(i,:) = lambda_mrw'; 
        store_u(i,:) = u'; 
        store_v(i,:) = v'; 
        lambda=(rho(2:T)-1).*rhoL./(rhoL-1); 
        lambda_mrw=rho(1:T); 
              
    dy=y(2:T)-y(1:T-1);
    
    end
    
    if ( mod( loop, 2000 ) ==0 )
        disp(  [ num2str( loop ) ' loops... ' ] )
    end 
    
end

disp( ['MCMC takes '  num2str( etime( clock, start_time) ) ' seconds' ] );
disp(' ' );

%% plot graphs
tauhat = mean(store_tau)';
sorttau = sort(store_tau);
taulb = sorttau(ceil((nloop-burnin)*.16),:)';
tauub = sorttau(ceil((nloop-burnin)*.84),:)';
hhat = mean(store_h)'; 
sorth = sort(store_h);

hlb = sorth(ceil((nloop-burnin)*.16),:)';
hub = sorth(ceil((nloop-burnin)*.84),:)';
rhohat = mean(store_rho)';
% MRW insersions to Chan code
lambdahat = mean(store_lambda)'; 
sortlambda = sort(store_lambda); 
lambdalb = sortlambda(ceil((nloop-burnin)*.16),:)'; 
lambdaub = sortlambda(ceil((nloop-burnin)*.84),:)'; 
sortlambda_mrw = sort(store_lambda_mrw); 
lambdalb_mrw = sortlambda_mrw(ceil((nloop-burnin)*.16),:)'; 
lambdaub_mrw = sortlambda_mrw(ceil((nloop-burnin)*.84),:)'; 
sortrho = sort(store_rho);
rholb = sortrho(ceil((nloop-burnin)*.16),:)';
rhoub = sortrho(ceil((nloop-burnin)*.84),:)';



% Part 2: MRW calculations

% These matrices are taken from Chan, Koop & Potter model, as estimated above
sigsqomega=exp(store_h);
sigsqtau=store_sig(:,1);
rho=store_rho;

% create sigsqeps, lambda, theta
% start with initial values
theta=zeros(nloop-burnin,T);
thetaalt=zeros(nloop-burnin,T);
lambda=zeros(nloop-burnin,T);
sigsqeps=zeros(nloop-burnin,T);
sigsqepsalt=zeros(nloop-burnin,T);
sigsqy=zeros(nloop-burnin,T);
sigsqc=zeros(nloop-burnin,T);
at=zeros(nloop-burnin,T);
bt=zeros(nloop-burnin,T);
rsqf=zeros(nloop-burnin,T);
rsqn=zeros(nloop-burnin,T);
sigsqeta=zeros(nloop-burnin,T);
gamma=zeros(nloop-burnin,T);
s=ones(nloop-burnin,T);
% time invariant 
thetati=zeros(nloop-burnin,T);
rsqfti=zeros(nloop-burnin,T);
rsqnti=zeros(nloop-burnin,T);

% make time invariant values
sigsqtauk=kron(ones(T,1),sigsqtau');
autocti=-sigsqomega./(sigsqtauk' + 2.*sigsqomega);
thetati=-( 1 - sqrt(1-4.*autocti.^2) )./ (2.*autocti);
lambdati=rho;
rsqfti=((thetati-lambdati).^2)./(1-lambdati.^2+(thetati-lambdati).^2);
rsqnti=1-thetati.^2.*(1-rsqfti);

% make initial  values
a1=sigsqomega(:,1)+lambdati(:,1).*sigsqtau;
b1=2*sigsqomega(:,1)+(1+lambdati(:,1).^2).*sigsqtau;
at=-a1.*ones(nloop-burnin,T);
bt=b1.*ones(nloop-burnin,T);

% make at and bt and lambda from recursion and theta and sigeps
lambda=rho;

ii=2;

while ii<=T;
    
lambda(:,ii)=rho(:,ii-1).*(rho(:,ii)-1)./(rho(:,ii-1)-1)    ;
    
at(:,ii)=(rho(:,ii)-lambda(:,ii)-1).*sigsqomega(:,ii-1)-lambda(:,ii).*sigsqtau;
bt(:,ii)=sigsqomega(:,ii)+sigsqtau.*(1+lambda(:,ii).^2)+ ((rho(:,ii)-lambda(:,ii)-1).^2).*sigsqomega(:,ii-1);
ii=ii+1;
end

% initialise sigsqc sigsqy and sigsqeps
sigsqc=zeros(nloop-burnin,T);    
sigsqy=zeros(nloop-burnin,T);
theta=thetati;
thetaalt=thetati;
sigsqc(:,1)=sigsqomega(:,1)./(1-(rho(:,1).^2)); % initial variance of cyclical component
sigsqc=sigsqc(:,1).*ones(nloop-burnin,T);

sigsqy(:,1)= sigsqtau + sigsqomega(:,1)+((rho(:,1)-1).^2).*sigsqc(:,1); % initial value of sigy
sigsqy=sigsqy(:,1).*ones(nloop-burnin,T);

sigsqyarma=sigsqy;
sigsqynfarma=sigsqy;
sigsqeps(:,1)=((1-lambda(:,1).^2)./(1-lambda(:,1).^2 + (theta(:,1) - lambda(:,1)).^2)).*sigsqy(:,1);
sigsqeps=sigsqeps(:,1).*ones(nloop-burnin,T);
sigsqepsalt=sigsqeps;

jj=2;
while jj<=T;

sigsqc(:,jj)=(rho(:,jj).^2).*sigsqc(:,jj-1) + sigsqomega(:,jj);
sigsqy(:,jj)=sigsqtau(:,1)+sigsqomega(:,jj)+((rho(:,jj)-1).^2).*sigsqc(:,jj-1);

theta(:,jj)=-at(:,jj)./sigsqeps(:,jj-1);
sigsqeps(:,jj)=bt(:,jj)-(theta(:,jj).^2).*sigsqeps(:,jj-1);
sigsqyarma(:,jj)=(lambda(:,jj).^2).*sigsqyarma(:,jj-1) ...
+((theta(:,jj).^2)-2.*lambda(:,jj).*theta(:,jj)).*sigsqeps(:,jj-1) + sigsqeps(:,jj);

sigsqepsalt(:,jj)=bt(:,jj)-(at(:,jj)).^2./sigsqepsalt(:,jj-1);

jj=jj+1;
end

jj=2;
while jj<=T;
    thetaalt(:,jj)=-at(:,jj)./sigsqepsalt(:,jj-1);
jj=jj+1;
end;

rsqf=1-(sigsqeps./sigsqy);
rsq=1-(sigsqomega+sigsqtauk')./sigsqy;

sigsqyti= sigsqtau + sigsqomega(:,T)+((rho(:,T)-1).^2).*sigsqc(:,T); % terminal value of sigy
sigsqepsti(:,T)=((1-lambdati(:,T).^2)./(1-lambdati(:,T).^2 + (thetati(:,T) - lambdati(:,T)).^2)).*sigsqyti;

% TERMINAL value for sigsqeta and gamma
sigsqeta(:,T)=(theta(:,T).^2).*sigsqeps(:,T);
    eee3=theta(:,T)>=1;
    sigsqeta(eee3,T)=(thetati(eee3,T).^2).*sigsqepsti(eee3,T);

jj=1;
while jj<T;

sigsqeta(:,T-jj)=(at(:,T-jj+1).^2)./(bt(:,T-jj+1)-sigsqeta(:,T-jj+1));    

jj=jj+1;
end

jj=2;
while jj<=T;

gamma(:,jj)=-at(:,jj)./sigsqeta(:,jj-1);    

jj=jj+1;
end

rsqn=1-(sigsqeta./sigsqy);

jj=2;
while jj<=T;

sigsqynfarma(:,jj)=(lambda(:,jj).^2).*sigsqynfarma(:,jj-1) ...
+((gamma(:,jj).^2)-2.*lambda(:,jj).*gamma(:,jj)).*sigsqeta(:,jj-1) + sigsqeta(:,jj);

jj=jj+1;
end

% some checks 
checkateps=at;
checkateta=at;
checkbteps=bt;
checkbteta=bt;

checkateps(:,2:end)=-theta(:,2:end).*sigsqeps(:,1:end-1);
checkateta(:,2:end)=-gamma(:,2:end).*sigsqeta(:,1:end-1);
checkbteps(:,2:end)=sigsqeps(:,2:end)+(theta(:,2:end).^2).*sigsqeps(:,1:end-1);
checkbteta(:,2:end)=sigsqeta(:,2:end)+(gamma(:,2:end).^2).*sigsqeta(:,1:end-1);

sorttheta = sort(theta);
thetalb = sorttheta(ceil((nloop-burnin)*.16),:)';
thetaub = sorttheta(ceil((nloop-burnin)*.84),:)';
thetahat = sorttheta(ceil((nloop-burnin)*.5),:)'; 

sortgamma = sort(gamma);
gammalb = sortgamma(ceil((nloop-burnin)*.16),:)';
gammaub = sortgamma(ceil((nloop-burnin)*.84),:)';
gammahat = sortgamma(ceil((nloop-burnin)*.5),:)'; 


sortrsqnti = sort(rsqnti);
rsqntilb = sortrsqnti(ceil((nloop-burnin)*.16),:)';
rsqntiub = sortrsqnti(ceil((nloop-burnin)*.84),:)';
rsqntihat = sortrsqnti(ceil((nloop-burnin)*.5),:)'; 

sortrsqfti = sort(rsqfti);
rsqftilb = sortrsqfti(ceil((nloop-burnin)*.16),:)';
rsqftiub = sortrsqfti(ceil((nloop-burnin)*.84),:)';
rsqftihat = sortrsqfti(ceil((nloop-burnin)*.5),:)'; 

sortthetati = sort(thetati);
thetatilb = sortthetati(ceil((nloop-burnin)*.16),:)';
thetatiub = sortthetati(ceil((nloop-burnin)*.84),:)';
thetatihat = sortthetati(ceil((nloop-burnin)*.5),:)'; 

sortlambda = sort(lambda);
lambdalb = sortlambda(ceil((nloop-burnin)*.16),:)';
lambdaub = sortlambda(ceil((nloop-burnin)*.84),:)';
lambdahat = sortlambda(ceil((nloop-burnin)*.5),:)'; 

sortlambdati = sort(lambdati);
lambdatilb = sortlambdati(ceil((nloop-burnin)*.16),:)';
lambdatiub = sortlambdati(ceil((nloop-burnin)*.84),:)';
lambdatihat = sortlambdati(ceil((nloop-burnin)*.5),:)'; 

sortrho = sort(rho);
rholb = sortrho(ceil((nloop-burnin)*.16),:)';
rhoub = sortrho(ceil((nloop-burnin)*.84),:)';
rhohat = sortrho(ceil((nloop-burnin)*.5),:)'; 

sortsigsqomega = sort(sigsqomega);
sigsqomegalb = sortsigsqomega(ceil((nloop-burnin)*.16),:)';
sigsqomegaub = sortsigsqomega(ceil((nloop-burnin)*.84),:)';
sigsqomegahat = sortsigsqomega(ceil((nloop-burnin)*.5),:)'; 

sortsigsqtau = sort(sigsqtauk');
sigsqtaulb = sortsigsqtau(ceil((nloop-burnin)*.16),:)';
sigsqtauub = sortsigsqtau(ceil((nloop-burnin)*.84),:)';
sigsqtauhat = sortsigsqtau(ceil((nloop-burnin)*.5),:)'; 

dif_r2=rsqn-rsqf;
sortdif_r2 = sort(dif_r2);
dif_r2lb = sortdif_r2(ceil((nloop-burnin)*.16),:)';
dif_r2ub = sortdif_r2(ceil((nloop-burnin)*.84),:)';
dif_r2hat = sortdif_r2(ceil((nloop-burnin)*.5),:)'; 


sortr2min = sort(rsqf);
r2minlb = sortr2min(ceil((nloop-burnin)*.16),:)';
r2minub = sortr2min(ceil((nloop-burnin)*.84),:)';
r2minhat = sortr2min(ceil((nloop-burnin)*.5),:)'; 

sortr2max = sort(rsqn);
r2maxlb = sortr2max(ceil((nloop-burnin)*.16),:)';
r2maxub = sortr2max(ceil((nloop-burnin)*.84),:)';
r2maxhat = sortr2max(ceil((nloop-burnin)*.5),:)'; 

sortr2 = sort(rsq);
r2lb = sortr2(ceil((nloop-burnin)*.16),:)';
r2ub = sortr2(ceil((nloop-burnin)*.84),:)';
r2hat = sortr2(ceil((nloop-burnin)*.5),:)'; 

hh1=figure
 subplot(2,1,1); 
    hold on 

plot(tid(1:T), [r2maxhat r2minhat],':', 'LineWidth',1 ,'Color','black');
 xlim([1961 2017]); axis([-inf inf 0 1]);   title('R^2_{max} and R^2_{min}'); 
     
    subplot(2,1,2); 
    plot(tid(1:T), [dif_r2lb dif_r2hat dif_r2ub],':', 'LineWidth',1 ,'Color','black');
 xlim([1961 2017]);  axis([-inf inf 0 1]); title('16.5%, 50% and 83.5% quantiles of the posterior distribution of R^2_{max}- R^2_{min}'); 
 saveas(hh1,sprintf('R2_x12%d.png',priori)); 
 hold off  

    hh2=figure
subplot(2,1,1);
    hold on 
plot(tid(1:T), [thetahat thetaub thetalb],':', 'LineWidth',1 ,'Color','black');
 xlim([1961 2017]);   title('16.5%, 50% and 83.5% quantiles of the posterior distribution of \theta_t'); 
 
   subplot(2,1,2); 
    hold on 
plot(tid(1:T), [thetahat lambdahat rhohat],':', 'LineWidth',1 ,'Color','black');
 xlim([1961 2017]);  title('\theta_t, \lambda_t and \rho_t' ); 
    hold off 
saveas(hh2,sprintf('theta_x12%d.png',priori)); 

% dump of matrices/results - for further analysis
dump=[tauhat rhohat thetahat hhat lambdahat r2minhat r2maxhat r2hat sigsqomegahat sigsqtauhat y ...
        thetatihat lambdatihat gammahat thetalb thetaub dif_r2lb dif_r2hat dif_r2ub];

filename = 'junk.xls';
col_header = {'tauhat', 'rhohat', 'thetahat', 'hhat', 'lambdahat', 'r2minhat', 'r2maxhat', 'r2hat', 'sigsqomegahat', ...
'sigsqtauhat', 'y', 'thetatihat', 'lambdatihat', 'gammahat', 'thetalb', 'thetaub', 'dif_r2lb', 'dif_r2hat', 'dif_r2ub'};
sheet = priori;
xlRange = 'B5:T229';
xlswrite(filename,dump,sheet,xlRange);
xlswrite(filename,col_header,sheet,'B4:T4');

end
