% Code for Mitchell, Robertson and Wright: the "Stock-Watson-Chan" UC model.

% This code is in 2 parts. 

% Part 1 takes, with the kind permission of Joshua
% Chan, Chan's code (also available at
% http://joshuachan.org/code/code_spectest.html), and estimates what we
% call the "SWC" model. 
% Minor edits have been made to Chan's code, indicated below, to work it through our data; 
% these edits and any ensuing errors are the responsibility of Mitchell, 
% Robertson and Wright

% Part 2 then uses the "output" from the SWC model, as described in
% Mitchell, Robertson and Wright.


% Part 1 (Chan's code and words.) This is the main run file for testing 
% if stochastic volatility is needed
% in an unobserved components model for modeling inflation in G7 countries.
% See the first application in Chan (2016)
%
% This code is free to use for academic purposes only, provided that the 
% paper is cited as:
%
% Chan, J.C.C. (2016). Specification Tests for Time-Varying Parameter 
% Models with Stochastic Volatility, Econometric Reviews, forthcoming
%
% This code comes without technical support of any kind.  It is expected to
% reproduce the results reported in the paper. Under no circumstances will
% the authors be held responsible for any use (or misuse) of this code in
% any way.


clear; clc;
R = 5;
nloop = 40000;
burnin = 10000;

% Load in MRW data for CPI inflation for the 8 OECD countries. These are
% seasonally adjusted data. Seasonal adjustment is via X12 as implemented
% in EViews 9.5. For the US, SA data are available not just as adjusted by X12
% but as published directly in FRED. 
data = xlsread('SA inflation Eviews 7 countries x12.xlsx', 'B2:j226'); 

for countryi = 9  % choose the series/country to analyse
country_id = 9; % 1: Germany, 2= France; 3=GB; 4= Greece, 5-Italy; 6=Japan,
%7=US (this series is from FRED and SA via 12), 8= Canada 9=US SA directly
%by FRED not x12

y = data(1:end,countryi)*100;

tid = (1961:.25:2017)';
startd=1961;

disp('Starting MCMC.... ');
disp(' ' );
start_time = clock;    

store_lBF = zeros(R,3);
for bigloop = 1:R
    disp(  [ num2str( R-bigloop+1) ' big loops to go... ' ] )
    UCSV_gam;
    theta_qan = quantile(store_theta,[.05 .95]);
    store_lBF(bigloop,:) = lBF;        
end
disp( ['MCMC takes '  num2str( etime( clock, start_time) ) ' seconds' ] );
disp(' ' );
BFhat = mean(store_lBF)';
BFstd = std(store_lBF)/sqrt(R);

% report estimates
fprintf('\n'); 
fprintf('Parameter   | Posterior mean (Posterior std. dev.):\n'); 
fprintf('omega2_h    | %.2f (%.2f)\n', thetahat(1), thetastd(1)); 
fprintf('omega2_g    | %.2f (%.2f)\n', thetahat(2), thetastd(2)); 

% report log BFs
fprintf('\n'); 
fprintf('log Bayes Factor | estimate (NSE):\n'); 
fprintf('log BF_uh        | %.1f (%.2f)\n', BFhat(1), BFstd(1)); 
fprintf('log BF_ug        | %.1f (%.2f)\n', BFhat(2), BFstd(2)); 
fprintf('log BF_u,gh      | %.1f (%.2f)\n', BFhat(3), BFstd(3)); 

% plot some figures
figure;
subplot(1,2,1);
hold on
    plot(omh_grid,pomhhat,'LineWidth',1,'Color','blue');
    plot(omh_grid,priden_omh,'--','LineWidth',1,'Color','red');
hold off
title('\omega_h');
subplot(1,2,2);
hold on
    plot(omg_grid,pomghat,'LineWidth',1,'Color','blue'); 
    plot(omg_grid,priden_omg,'--','LineWidth',1,'Color','red');
hold off
box off;% legend('posterior','prior',1);
title('\omega_g');
set(gcf,'Position',[100 100 800 300]);

% plot h and g
figure
subplot(1,2,1); 
hold on 
    plotCI(tid,hCI(:,1),hCI(:,2)); 
    plot(tid,hhat,'LineWidth',1,'Color','blue');     
hold off
xlim([tid(1)-.5,tid(end)+.5]); box off;
title('exp(h_t/2)');
subplot(1,2,2); 
hold on 
    plotCI(tid,gCI(:,1),gCI(:,2)); 
    plot(tid,ghat,'LineWidth',1,'Color','blue');     
hold off
xlim([tid(1)-.5,tid(end)+.5]); box off;
title('exp(g_t/2)');
set(gcf,'Position',[100 100 800 300]);


% Part 2: MRW calculations

% These matrices are taken from Chan UCSV model, as estimated above
sigsqomega=((exp(store_h/2)).^2); % transitory
sigsqtau=((exp(store_g/2)).^2); % permanent
store_rho=zeros(nloop-burnin,T);  
rho=store_rho;

% start with initial values
theta=zeros(nloop-burnin,T);
thetaalt=zeros(nloop-burnin,T);
lambda=zeros(nloop-burnin,T);
sigsqeps=zeros(nloop-burnin,T);
sigsqepsalt=zeros(nloop-burnin,T);
sigsqy=zeros(nloop-burnin,T);
sigsqc=zeros(nloop-burnin,T);
at=zeros(nloop-burnin,T);
bt=zeros(nloop-burnin,T);
rsqf=zeros(nloop-burnin,T);
rsqn=zeros(nloop-burnin,T);
sigsqeta=zeros(nloop-burnin,T);
gamma=zeros(nloop-burnin,T);
s=ones(nloop-burnin,T);
% time invariant 
thetati=zeros(nloop-burnin,T);
rsqfti=zeros(nloop-burnin,T);
rsqnti=zeros(nloop-burnin,T);

% make time invariant values
autocti=-sigsqomega./(sigsqtau + 2.*sigsqomega);
thetati=-( 1 - sqrt(1-4.*autocti.^2) )./ (2.*autocti);
lambdati=rho;
rsqfti=((thetati-lambdati).^2)./(1-lambdati.^2+(thetati-lambdati).^2);
rsqnti=1-thetati.^2.*(1-rsqfti);

% make initial  values
a1=sigsqomega(:,1)+lambdati(:,1).*sigsqtau(:,1);
b1=2*sigsqomega(:,1)+(1+lambdati(:,1).^2).*sigsqtau(:,1);
at=-a1.*ones(nloop-burnin,T);
bt=b1.*ones(nloop-burnin,T);

% make at and bt and lambda from recursion and theta and sigeps
lambda=rho;
lambdati=rho;

ii=2;

while ii<=T;
    
lambda(:,ii)=rho(:,ii-1).*(rho(:,ii)-1)./(rho(:,ii-1)-1)    ;
    
at(:,ii)=(rho(:,ii)-lambda(:,ii)-1).*sigsqomega(:,ii-1)-lambda(:,ii).*sigsqtau(:,ii-1);
bt(:,ii)=sigsqomega(:,ii)+sigsqtau(:,ii)+(lambda(:,ii).^2).*sigsqtau(:,ii-1)+ ((rho(:,ii)-lambda(:,ii)-1).^2).*sigsqomega(:,ii-1);
ii=ii+1;
end

% initialise sigsqc sigsqy and sigsqeps

sigsqc=zeros(nloop-burnin,T);    
sigsqy=zeros(nloop-burnin,T);
theta=thetati;
thetaalt=thetati;
sigsqc(:,1)=sigsqomega(:,1)./(1-(rho(:,1).^2)); % initial variance of cyclical component

sigsqy(:,1)= sigsqtau(:,1) + sigsqomega(:,1)+((rho(:,1)-1).^2).*sigsqc(:,1); % initial value of sigy
sigsqyarma=sigsqy;
sigsqynfarma=sigsqy;
sigsqeps(:,1)=((1-lambda(:,1).^2)./(1-lambda(:,1).^2 + (theta(:,1) - lambda(:,1)).^2)).*sigsqy(:,1);
sigsqepsalt=sigsqeps;

jj=2;
while jj<=T;

sigsqc(:,jj)=(rho(:,jj).^2).*sigsqc(:,jj-1) + sigsqomega(:,jj);
sigsqy(:,jj)=sigsqtau(:,jj)+sigsqomega(:,jj)+((rho(:,jj)-1).^2).*sigsqc(:,jj-1);

theta(:,jj)=-at(:,jj)./sigsqeps(:,jj-1);
sigsqeps(:,jj)=bt(:,jj)-(theta(:,jj).^2).*sigsqeps(:,jj-1);
sigsqyarma(:,jj)=(lambda(:,jj).^2).*sigsqyarma(:,jj-1) ...
+((theta(:,jj).^2)-2.*lambda(:,jj).*theta(:,jj)).*sigsqeps(:,jj-1) + sigsqeps(:,jj);

sigsqepsalt(:,jj)=bt(:,jj)-(at(:,jj)).^2./sigsqepsalt(:,jj-1);

jj=jj+1;
end

jj=2;
while jj<=T;
    thetaalt(:,jj)=-at(:,jj)./sigsqepsalt(:,jj-1);
jj=jj+1;
end;

rsqf=1-(sigsqeps./sigsqy);
rsq=1-(sigsqomega+sigsqtau)./sigsqy;


sigsqyti= sigsqtau + sigsqomega(:,T)+((rho(:,T)-1).^2).*sigsqc(:,T); % terminal value of sigy
sigsqepsti=((1-lambdati(:,T).^2)./(1-lambdati(:,T).^2 + (thetati(:,T) - lambdati(:,T)).^2)).*sigsqyti(:,T);


% terminal value for sigsqeta and gamma
sigsqeta(:,T)=(theta(:,T).^2).*sigsqeps(:,T);

% TERMINAL value switch for sigsqeta and gamma
sigsqeta(:,T)=(theta(:,T).^2).*sigsqeps(:,T);
    eee3=theta(:,T)>=1;
    sigsqeta(eee3,T)=(thetati(eee3,T).^2).*sigsqepsti(eee3,1);


jj=1;
while jj<T;

sigsqeta(:,T-jj)=(at(:,T-jj+1).^2)./(bt(:,T-jj+1)-sigsqeta(:,T-jj+1));    

jj=jj+1;
end

jj=2;
while jj<=T;

gamma(:,jj)=-at(:,jj)./sigsqeta(:,jj-1);    

jj=jj+1;
end

rsqn=1-(sigsqeta./sigsqy);

jj=2;
while jj<=T;

sigsqynfarma(:,jj)=(lambda(:,jj).^2).*sigsqynfarma(:,jj-1) ...
+((gamma(:,jj).^2)-2.*lambda(:,jj).*gamma(:,jj)).*sigsqeta(:,jj-1) + sigsqeta(:,jj);

jj=jj+1;
end

% some checks 
checkateps=at;
checkateta=at;
checkbteps=bt;
checkbteta=bt;

checkateps(:,2:end)=-theta(:,2:end).*sigsqeps(:,1:end-1);
checkateta(:,2:end)=-gamma(:,2:end).*sigsqeta(:,1:end-1);
checkbteps(:,2:end)=sigsqeps(:,2:end)+(theta(:,2:end).^2).*sigsqeps(:,1:end-1);
checkbteta(:,2:end)=sigsqeta(:,2:end)+(gamma(:,2:end).^2).*sigsqeta(:,1:end-1);

sorttheta = sort(theta);
thetalb = sorttheta(ceil((nloop-burnin)*.16),:)';
thetaub = sorttheta(ceil((nloop-burnin)*.84),:)';
thetahat = sorttheta(ceil((nloop-burnin)*.5),:)'; 

sortlambda = sort(lambda);
lambdalb = sortlambda(ceil((nloop-burnin)*.16),:)';
lambdaub = sortlambda(ceil((nloop-burnin)*.84),:)';
lambdahat = sortlambda(ceil((nloop-burnin)*.5),:)'; 

sortrho = sort(rho);
rholb = sortrho(ceil((nloop-burnin)*.16),:)';
rhoub = sortrho(ceil((nloop-burnin)*.84),:)';
rhohat = sortrho(ceil((nloop-burnin)*.5),:)'; 

sortlambdati = sort(lambdati);
lambdatilb = sortlambdati(ceil((nloop-burnin)*.16),:)';
lambdatiub = sortlambdati(ceil((nloop-burnin)*.84),:)';
lambdatihat = sortlambdati(ceil((nloop-burnin)*.5),:)'; 


sortthetati = sort(thetati);
thetatilb = sortthetati(ceil((nloop-burnin)*.16),:)';
thetatiub = sortthetati(ceil((nloop-burnin)*.84),:)';
thetatihat = sortthetati(ceil((nloop-burnin)*.5),:)'; 


sortsigsqomega = sort(sigsqomega);
sigsqomegalb = sortsigsqomega(ceil((nloop-burnin)*.16),:)';
sigsqomegaub = sortsigsqomega(ceil((nloop-burnin)*.84),:)';
sigsqomegahat = sortsigsqomega(ceil((nloop-burnin)*.5),:)'; 

sortsigsqtau = sort(sigsqtau);
sigsqtaulb = sortsigsqtau(ceil((nloop-burnin)*.16),:)';
sigsqtauub = sortsigsqtau(ceil((nloop-burnin)*.84),:)';
sigsqtauhat = sortsigsqtau(ceil((nloop-burnin)*.5),:)'; 

dif_r2=rsqn-rsqf;
sortdif_r2 = sort(dif_r2);
dif_r2lb = sortdif_r2(ceil((nloop-burnin)*.16),:)';
dif_r2ub = sortdif_r2(ceil((nloop-burnin)*.84),:)';
dif_r2hat = sortdif_r2(ceil((nloop-burnin)*.5),:)'; 

sortgamma = sort(gamma);
gammalb = sortgamma(ceil((nloop-burnin)*.16),:)';
gammaub = sortgamma(ceil((nloop-burnin)*.84),:)';
gammahat = sortgamma(ceil((nloop-burnin)*.5),:)'; 

sortr2min = sort(rsqf);
r2minlb = sortr2min(ceil((nloop-burnin)*.16),:)';
r2minub = sortr2min(ceil((nloop-burnin)*.84),:)';
r2minhat = sortr2min(ceil((nloop-burnin)*.5),:)'; 

sortr2max = sort(rsqn);
r2maxlb = sortr2max(ceil((nloop-burnin)*.16),:)';
r2maxub = sortr2max(ceil((nloop-burnin)*.84),:)';
r2maxhat = sortr2max(ceil((nloop-burnin)*.5),:)'; 

sortr2 = sort(rsq);
r2lb = sortr2(ceil((nloop-burnin)*.16),:)';
r2ub = sortr2(ceil((nloop-burnin)*.84),:)';
r2hat = sortr2(ceil((nloop-burnin)*.5),:)'; 

figure
 subplot(2,1,1); 
    hold on 
axis([-inf inf 0 1])
plot(tid(1:T), [r2maxhat r2minhat],':', 'LineWidth',1 ,'Color','black');

xlim([1961 2017]);axis([-inf inf 0 1]);   title('R^2_{max} and R^2_{min}'); 
     
    
    subplot(2,1,2); 
     axis([-inf inf 0 1])
    plot(tid(1:T), [dif_r2lb dif_r2hat dif_r2ub],':', 'LineWidth',1 ,'Color','black');
 xlim([1961 2017]);  axis([-inf inf 0 1]);  title('16.5%, 50% and 83.5% quantiles of the posterior distribution of R^2_{max}- R^2_{min}'); 
    hold off  

    figure
subplot(2,1,1); 
    hold on 
plot(tid(1:T), [thetahat thetaub thetalb],':', 'LineWidth',1 ,'Color','black');
 xlim([1961 2017]);   title('16.5%, 50% and 83.5% quantiles of the posterior distribution of \theta_t'); 
     
   subplot(2,1,2); 
    hold on 
plot(tid(1:T), [thetahat lambdahat rhohat],':', 'LineWidth',1 ,'Color','black');
 xlim([1961 2017]);   title('\theta_t, \lambda_t and \rho_t' );
    hold off 

% dump of matrices/results - for further analysis
dump=[tauhat rhohat thetahat hhat lambdahat r2minhat r2maxhat r2hat sigsqomegahat sigsqtauhat y];
dump=[tauhat rhohat thetahat hhat lambdahat r2minhat r2maxhat r2hat sigsqomegahat sigsqtauhat y ...
        thetatihat lambdatihat gammahat thetalb thetaub dif_r2lb dif_r2hat dif_r2ub];


filename = 'junkSWC.xls';
col_header = {'tauhat', 'rhohat', 'thetahat', 'hhat', 'lambdahat', 'r2minhat', 'r2maxhat', 'r2hat', 'sigsqomegahat', ...
'sigsqtauhat', 'y', 'thetatihat', 'lambdatihat', 'gammahat', 'thetalb', 'thetaub', 'dif_r2lb', 'dif_r2hat', 'dif_r2ub'};
sheet = countryi;
xlRange = 'B5:T229';
xlswrite(filename,dump,sheet,xlRange);
xlswrite(filename,col_header,sheet,'B4:T4');

end
    
    